﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using DocumentFormat.OpenXml.Packaging;
using OpenXmlPowerTools;

class Program
{
    private static object FlattenParagraphsTransform(XNode node)
    {
        XElement element = node as XElement;
        if (element != null)
        {
            if (element.Name == W.p)
            {
                return element.Elements().Where(e => e.Name != W.pPr).Select(e => FlattenParagraphsTransform(e))
                    .Concat(
                        new[]
                        {
                            new XElement(W.p,
                                element.Attributes(),
                                element.Elements(W.pPr))
                        });
            }
            return new XElement(element.Name,
                element.Attributes(),
                element.Nodes().Select(n => FlattenParagraphsTransform(n)));
        }
        return node;
    }

    static string GetBookmarkText(WordprocessingDocument doc, string bookmarkName)
    {
        XDocument xDoc = doc.MainDocumentPart.GetXDocument();
        bool containsBookmark = xDoc.Descendants(W.bookmarkStart)
            .Where(d => (string)d.Attribute(W.name) == bookmarkName)
            .Any();
        if (!containsBookmark)
        {
            Console.WriteLine("Document doesn't contain bookmark.");
            Console.WriteLine("Throw an exception or take appropriate action.");
            Environment.Exit(0);
        }
        XElement newRoot = (XElement)FlattenParagraphsTransform(xDoc.Root);

        XElement startBookmarkElement = newRoot.Descendants(W.bookmarkStart)
            .Where(d => (string)d.Attribute(W.name) == bookmarkName)
            .FirstOrDefault();
        int bookmarkId = (int)startBookmarkElement.Attribute(W.id);
        XElement endBookmarkElement = newRoot.Descendants(W.bookmarkEnd)
            .Where(d => (int)d.Attribute(W.id) == bookmarkId)
            .FirstOrDefault();
        if (startBookmarkElement.Parent != endBookmarkElement.Parent)
        {
            Console.WriteLine("Bookmark start and end not at same levels.  Can't retrieve text");
            Environment.Exit(0);
        }

        XElement parentElement = startBookmarkElement.Parent;
        var elementsBetweenBookmarks = startBookmarkElement
            .ElementsAfterSelf()
            .TakeWhile(e => e != endBookmarkElement);
        var text = elementsBetweenBookmarks
            .Select(e =>
                {
                    if (e.Name == W.r)
                    {
                        string runText = e.Descendants(W.t).Select(t => (string)t).StringConcatenate();
                        return runText;
                    }
                    if (e.Name == W.p)
                        return Environment.NewLine;
                    return "";
                })
            .StringConcatenate();
        return text;
    }

    static void Main(string[] args)
    {
        File.Delete("../../Test01out.docx");
        File.Copy("../../Test01.docx", "../../Test01out.docx");
        using (WordprocessingDocument doc =
            WordprocessingDocument.Open("../../Test01out.docx", true))
        {
            Console.WriteLine("Test01.docx Bookmark A: >{0}<", GetBookmarkText(doc, "A"));
        }
        File.Delete("../../Test02out.docx");
        File.Copy("../../Test02.docx", "../../Test02out.docx");
        using (WordprocessingDocument doc =
            WordprocessingDocument.Open("../../Test02out.docx", true))
        {
            Console.WriteLine("Test02.docx Bookmark B: >{0}<", GetBookmarkText(doc, "B"));
        }
        File.Delete("../../Test03out.docx");
        File.Copy("../../Test03.docx", "../../Test03out.docx");
        using (WordprocessingDocument doc =
            WordprocessingDocument.Open("../../Test03out.docx", true))
        {
            Console.WriteLine("Test03.docx Bookmark C: >{0}<", GetBookmarkText(doc, "C"));
        }
        File.Delete("../../Test04out.docx");
        File.Copy("../../Test04.docx", "../../Test04out.docx");
        using (WordprocessingDocument doc =
            WordprocessingDocument.Open("../../Test04out.docx", true))
        {
            Console.WriteLine("Test04.docx Bookmark ABC: >{0}<", GetBookmarkText(doc, "ABC"));
        }
    }
}
